# -*- coding: utf-8 -*-
#    Copyright (C) 2007 Stewart Adam
#    This file is part of audio-convert-mod.

#    audio-convert-mod is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.

#    audio-convert-mod is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with audio-convert-mod; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
Custom widgets.
"""

import gtk, gtk.glade, gobject
import logging
import pango
import types

import formats
from i18n import _
from const import *

class StatusBar(gtk.Statusbar):
    def __init__(self, widget):
        """ Initialize.
            widget: GTK widget to use as the statusbar
        """
        gtk.Statusbar.__init__(self)
        self.statusbar = widget

    def newmessage(self, message, seconds=3):
        """ New message in status bar.
            message: Message to display
            seconds: number of seconds to display the message
                     defaults to three.
        """
        try:
            self.message_timeout()
        # Just means there was no existing message.
        except AttributeError:
            pass
        self.statusbar.push(1, message)
        self.message_timer = gobject.timeout_add(seconds*1000, self.message_timeout)

    def message_timeout(self):
        """ Remove a message from the statusbar """
        self.statusbar.pop(1)
        gobject.source_remove(self.message_timer)
        return True

class ProgressBar(gtk.ProgressBar):
    def __init__(self, widget, ms=15):
        """ Initialize.
            widget: GTK widget to use as the progressbar
            ms: Pulse step is every this many miliseconds
        """
        gtk.ProgressBar.__init__(self)
        self.progressbar = widget
        self.ms = ms
        self.pulsing = False
        self.progressbar.set_pulse_step(0.01)

    def _pulse(self):
        """ Pulse the progressbar. """
        self.progressbar.pulse()
        # Has to return true to happen again
        return True

    def startPulse(self):
        """ Start auto-pulsing the progressbar """
        if self.pulsing:
            self.stopPulse()
        self.pulsing = True
        self.pulsetimer = gobject.timeout_add(self.ms, self._pulse)

    def stopPulse(self):
        """ Stop auto-pulsing the progressbar """
        gobject.source_remove(self.pulsetimer)
        self.progressbar.set_fraction(0)
        self.pulsing = False
        return True

    def setMs(self, ms):
        """ Set the ms value (see __init__) """ 
        self.ms = int(ms)
        if self.pulsing:
            self.stopPulse()
            self.startPulse()

class GenericDia:
    """ Wrapper for the generic dialog """
    def __init__(self, widget):
        """ Initialize """
        self.dialog = widget
    def run(self):
        """ Run the dialog """
        self.dialog.show()
        try:
            return self.dialog.run()
        except:
            self.destroy()
            raise
    def destroy(self):
        """ Hide the dialog, don't kill it! """
        self.dialog.hide()

class PathBrowser(GenericDia):
    """ Wrapper for path dialog """
    def __init__(self, widget, ffilter=None):
        """ Initialize.
            widget: The widget to use
            ffilter: List for filter creation.
                    > 1st value is filter pattern
                    > 2nd value is the description
        """
        GenericDia.__init__(self, widget)
        if ffilter:
            self.ffilter = gtk.FileFilter()
            self.ffilter.add_pattern(ffilter[0])
            self.ffilter.set_name(ffilter[1])
            self.dialog.add_filter(self.ffilter)
        else:
            self.ffilter = None

    def destroy(self):
        """ Destroy the dialog and the filter """
        GenericDia.destroy(self)
        if self.ffilter:
            self.dialog.remove_filter(self.ffilter)
    def set_current_folder(self, path):
        """ Wrapper: See GTK+ help. """
        return self.dialog.set_current_folder(path)
    def set_select_multiple(self, bool):
        """ Wrapper: See GTK+ help. """
        if bool == True:
            return self.dialog.set_select_multiple(True)
        elif bool == False:
            return self.dialog.set_select_multiple(False)
    def set_action(self, action):
        """ Wrapper: See GTK+ help. """
        return self.dialog.set_action(action)
    def set_title(self, title):
        """ Wrapper: See GTK+ help. """
        return self.dialog.set_title(title)
    def get_filename(self):
        """ Wrapper: See GTK+ help. """
        return self.dialog.get_filenames()
    def get_filenames(self):
        """ Wrapper: See GTK+ help. """
        return self.dialog.get_filenames()
    def set_filename(self, filename):
        """ Wrapper: See GTK+ help. """
        return self.dialog.set_filename(filename)
    def add_filter(self, ffilter):
        """ Wrapper: See GTK+ help. """
        return self.dialog.add_filter(ffilter)
    def remove_filter(self, ffilter):
        """ Wrapper: See GTK+ help. """
        return self.dialog.remove_filter(ffilter)


def addToSelectFilesFilesTreeview(self, paths):
  """ Adds a file to the Select Files treeview """
  model = self.ui.main1FilesTreeview.get_model()
  # don't sort as we add
  model.set_default_sort_func(lambda *args: -1) 
  # don't refresh as we add
  self.ui.main1FilesTreeview.freeze_child_notify()
  # detach model
  self.ui.main1FilesTreeview.set_model(None)
  errors = []
  for i in paths:
    fileType, path, filename = formats.getFileType(i), '/'.join(i.split('/')[:-1]), i.split('/')[-1]
    array, coord = audio_convert_mod.liststoreIndex(model, filename)
    if False == True: # coord != []:
      # a filename match was found, now compare paths
      array2, coord2 = audio_convert_mod.liststoreIndex(model, path)
      if coord2 != []:
        errors.append( _('Skipping duplicate file `%(a)s/%(b)s`' % {'a': array2[coord2[0][0]][coord2[0][1]], \
                                                                      'b': array[coord[0][0]][coord[0][1]]\
                                                                   } ) )
    else:
      # No duplicate because array[coord[0][0]][coord[0][1]] isn't there
      if formats.decodable(i) and not os.path.isdir(i):
        model.append([fileType.__class__.__name__, path, filename])
      else:
        errors.append( _('Skipping non-decodable file `%(a)s`' % {'a': i} ) )
  # okay, now you can sort.
  model.set_sort_column_id(-1, gtk.SORT_ASCENDING)
  # okay, now you can refresh.
  self.ui.main1FilesTreeview.thaw_child_notify()
  # attach model
  self.ui.main1FilesTreeview.set_model(model)
  return errors


def setupSelectFilesFilesTreeview(self):
  """ Well, the name pretty much explains it ;) """

  # Give it columns
  """cell = gtk.CellRendererPixbuf()
  col = gtk.TreeViewColumn(_('Decodable'), cell, stock_id=0)
  col.set_resizable(True)
  col.set_resizable(True)
  col.set_sort_column_id(0)
  self.ui.main1FilesTreeview.append_column(col)"""

  cell = gtk.CellRendererText()
  cell.set_property('ellipsize', pango.ELLIPSIZE_END)
  col = gtk.TreeViewColumn(_('Format'), cell, text=0)
  col.set_resizable(True)
  col.set_sort_column_id(0)
  col.set_min_width(75)
  self.ui.main1FilesTreeview.append_column(col)

  cell = gtk.CellRendererText()
  cell.set_property('ellipsize', pango.ELLIPSIZE_MIDDLE)
  col = gtk.TreeViewColumn(_('Path'), cell, text=1)
  col.set_resizable(True)
  col.set_sort_column_id(1)
  col.set_min_width(150)
  self.ui.main1FilesTreeview.append_column(col)

  cell = gtk.CellRendererText()
  cell.set_property('ellipsize', pango.ELLIPSIZE_START)
  col = gtk.TreeViewColumn(_('Filename'), cell, text=2)
  col.set_resizable(True)
  col.set_sort_column_id(2)
  self.ui.main1FilesTreeview.append_column(col)

  selectFilesListstore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
  self.ui.main1FilesTreeview.set_model(selectFilesListstore)
  selection = self.ui.main1FilesTreeview.get_selection()
  selection.set_mode(gtk.SELECTION_MULTIPLE)

  # Allow enable drag and drop of rows including row move
  #self.TARGETS = [('text/plain', 0, 1)]
  #self.treeview.enable_model_drag_dest(self.TARGETS, gtk.gdk.ACTION_DEFAULT)
  target = [('text/uri-list', 0, 0)]
  self.ui.main1FilesTreeview.drag_dest_set(gtk.DEST_DEFAULT_ALL, target, gtk.gdk.ACTION_COPY)
  # do this in self.: self.ui.main1FilesTreeview.connect('drag_data_received', drag_data_received)


def setupFeaturesTreeview(self):
  """ Well, the name pretty much explains it ;) """
  # Give it columns
  cell = gtk.CellRendererText()
  cell.set_property('ellipsize', pango.ELLIPSIZE_MIDDLE)
  col = gtk.TreeViewColumn(_('Format'), cell, text=0)
  col.set_resizable(False)
  col.set_sort_column_id(0)
  col.set_min_width(200)
  self.ui.featuresFeaturesTreeview.append_column(col)

  cell = gtk.CellRendererPixbuf()
  cell.set_property('stock-size', gtk.ICON_SIZE_BUTTON)
  col = gtk.TreeViewColumn(_('Decode'), cell, stock_id=1)
  col.set_resizable(False)
  col.set_sort_column_id(1)
  col.set_min_width(25)
  self.ui.featuresFeaturesTreeview.append_column(col)

  cell = gtk.CellRendererPixbuf()
  cell.set_property('stock-size', gtk.ICON_SIZE_BUTTON)
  col = gtk.TreeViewColumn(_('Encode'), cell, stock_id=2)
  col.set_resizable(False)
  col.set_sort_column_id(2)
  col.set_min_width(25)
  self.ui.featuresFeaturesTreeview.append_column(col)

  featuresListstore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
  self.ui.featuresFeaturesTreeview.set_model(featuresListstore)


